import { ClientSession, SubscriptionId } from "../client_session";
/**
 * A client side implementation to deal with publish service.
 *
 * @class ClientSidePublishEngine
 * The ClientSidePublishEngine encapsulates the mechanism to
 * deal with a OPCUA Server and constantly sending PublishRequest
 * The ClientSidePublishEngine also performs  notification acknowledgements.
 * Finally, ClientSidePublishEngine dispatch PublishResponse to the correct
 * Subscription id callback
 *
 * @param session {ClientSession} - the client session
 *
 *
 * @constructor
 */
export declare class ClientSidePublishEngine {
    static publishRequestCountInPipeline: number;
    timeoutHint: number;
    activeSubscriptionCount: number;
    nbPendingPublishRequests: number;
    nbMaxPublishRequestsAcceptedByServer: number;
    isSuspended: boolean;
    session: ClientSession | null;
    private subscriptionAcknowledgements;
    private readonly subscriptionMap;
    constructor(session: ClientSession);
    /**
     * the number of active subscriptions managed by this publish engine.
     * @property subscriptionCount
     * @type {Number}
     */
    get subscriptionCount(): number;
    suspend(suspendedState: boolean): void;
    acknowledge_notification(subscriptionId: SubscriptionId, sequenceNumber: number): void;
    cleanup_acknowledgment_for_subscription(subscriptionId: SubscriptionId): void;
    /**
     * @method send_publish_request
     */
    send_publish_request(): void;
    terminate(): void;
    registerSubscription(subscription: any): void;
    replenish_publish_request_queue(): void;
    /**
     * @method unregisterSubscription
     *
     * @param subscriptionId
     */
    unregisterSubscription(subscriptionId: SubscriptionId): void;
    getSubscriptionIds(): SubscriptionId[];
    /***
     * get the client subscription from Id
     */
    getSubscription(subscriptionId: SubscriptionId): any;
    hasSubscription(subscriptionId: SubscriptionId): boolean;
    republish(callback: () => void): void;
    internalSendPublishRequest(): void;
    private _receive_publish_response;
    private _republish;
    private __repairSubscription;
}
